# -*- coding: utf-8 -*-
from django.utils.translation import gettext_lazy as _
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from thx.api.mixins import ThuxModelMixin
from thx.role.core.views import ThuxRoleViewSetMixin

from ... import models
from . import serializers, filters, permissions

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)


# NOTIFICATION
class NotificationViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    NotificationViewSet
    -----------

    :action list: Return the notification list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given notification.
        `Profiles`: `superuser`, `staff`

    :action light: Return the notification list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new notification instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a notification instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a notification instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a notification instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Notification.objects.all()
    serializer_class = serializers.NotificationSerializer
    filter_backends = (filters.NotificationFilterBackend,)
    permission_classes = (permissions.NotificationPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a notification instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a notification instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# NOTIFICATION RECIPIENT
class NotificationRecipientViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    NotificationRecipientViewSet
    -----------

    :action list: Return the notification recipient list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given notification recipient.
        `Profiles`: `superuser`, `staff`

    :action light: Return the notification recipient list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new notification recipient instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a notification recipient instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a notification recipient instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a notification recipient instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.NotificationRecipient.objects.all()
    serializer_class = serializers.NotificationRecipientSerializer
    filter_backends = (filters.NotificationRecipientFilterBackend,)
    permission_classes = (permissions.NotificationRecipientPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a notification recipient instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a notification recipient instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# NOTIFICATION TYPE
class NotificationTypeViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    NotificationTypeViewSet
    -----------

    :action list: Return the notification type list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given notification type.
        `Profiles`: `superuser`, `staff`

    :action light: Return the notification type list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new notification type instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a notification type instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a notification type instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a notification type instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.NotificationType.objects.all()
    serializer_class = serializers.NotificationTypeSerializer
    filter_backends = (filters.NotificationTypeRecipientFilterBackend,)
    permission_classes = (permissions.NotificationTypeRecipientPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a notification type instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a notification type instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)
