# -*- coding: utf-8 -*-

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.exceptions import PermissionDenied
from django.db import models, transaction
from django.utils.translation import gettext_lazy as _
from thx.core.models import CleanModel, DateModel, OrderedModel, StatusModel, UserModel

User = get_user_model()


class Notification(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    type = models.ForeignKey(
        'NotificationType',
        blank=True, null=True,
        on_delete=models.CASCADE,
        related_name='notifications',
        verbose_name=_('type')
    )
    subject = models.CharField(
        max_length=20,
        verbose_name=_('subject'),
    )
    slug = models.SlugField(
        max_length=20,
        verbose_name=_('slug'),
    )
    body = models.TextField(
        verbose_name=_('body'),
    )
    call_to_action_label = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('call to action label'),
    )
    call_to_action_url = models.URLField(
        blank=True, null=True,
        verbose_name=_('call to action url'),
    )

    class Meta:
        verbose_name = _('notification')
        verbose_name_plural = _('notifications')
        ordering = ('ordering', '-date_create',)
        permissions = (
            ("list_notification", "Can list notification"),
            ("detail_notification", "Can detail notification"),
            ("enable_notification", "Can enable notification"),
            ("disable_notification", "Can disable notification"),
        )

    def __str__(self):
        return self.subject

    @classmethod
    @transaction.atomic
    def make_notification(
        cls,
        recipients,
        subject,
        slug,
        body,
        creator,
        call_to_action_label=None,
        call_to_action_url=None,
        batch_size=settings.THX_NOTIFY_BATCH_SIZE
    ):
        notification = cls.objects.create(
            subject=subject,
            slug=slug,
            body=body,
            call_to_action_label=call_to_action_label,
            call_to_action_url=call_to_action_url,
            creator=creator,
            last_modifier=creator,
        )
        notification_recipients = NotificationRecipient.objects.bulk_create(
            [
                NotificationRecipient(
                    user=user,
                    notification=notification,
                    reading_date=None,
                    creator=creator,
                    last_modifier=creator,
                ) for user in recipients
            ],
            batch_size=batch_size,
        )
        return notification, notification_recipients

    @property
    def type_name(self):
        return self.type.name if self.type else None


class NotificationRecipient(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    user = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        related_name='notifications',
        verbose_name=_('user'),
    )
    notification = models.ForeignKey(
        Notification,
        on_delete=models.CASCADE,
        related_name='notification_recipients',
        verbose_name=_('notification'),
    )
    reading_date = models.DateTimeField(
        blank=True, null=True,
        verbose_name=_('reading date')
    )

    class Meta:
        verbose_name = _('notification recipient')
        verbose_name_plural = _('notification recipients')
        ordering = ('ordering', '-notification__date_create')
        permissions = (
            ("list_notificationrecipient", "Can list notification recipient"),
            ("detail_notificationrecipient", "Can detail notification recipient"),
            ("enable_notificationrecipient", "Can enable notification recipient"),
            ("disable_notificationrecipient", "Can disable notification recipient"),
        )
        unique_together = ('user', 'notification')

    def __str__(self):
        return '{}-{}'.format(self.user, self.notification)

    @property
    def user_email(self):
        return self.user.email

    @property
    def notification_subject(self):
        return self.notification.subject


class NotificationType(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    code = models.CharField(
        max_length=50,
        verbose_name=_("code"),
    )
    category = models.CharField(
        max_length=3,
        choices=settings.NOTIFY_NOTIFICATIONTYPE_DEF,
        db_index=True,
        verbose_name=_("category"),
    )
    name = models.CharField(
        max_length=255,
        unique=True,
        verbose_name=_("name"),
    )
    description = models.TextField(
        blank=True,
        verbose_name=_("description"),
    )
    note = models.TextField(
        blank=True,
        verbose_name=_("note"),
    )
    periodicity = models.IntegerField(
        blank=True, null=True,
        verbose_name=_('periodicity in days')
    )

    class Meta:
        verbose_name = _('notification type')
        verbose_name_plural = _('notification types')
        ordering = ('ordering', 'name')
        permissions = (
            ("list_notificationtype", "Can list notification type"),
            ("detail_notificationtype", "Can detail notification type"),
            ("enable_notificationtype", "Can enable notification type"),
            ("disable_notificationtype", "Can disable notification type"),
        )

    def __str__(self):
        return self.name
